const assert = require('assert');
const { describe, it } = require('node:test');
const prepareRoutes = require('../lib/prepare-routes');
const { join } = require('path');
const { outputFileSync, removeSync } = require('fs-extra')
const testDir = join(__dirname, 'testFiles')

describe('prepareRoutes', () => {

   it('test', () => {
      outputFileSync(join(testDir, 'valid.js'), 'module.exports = () => {}', 'utf-8')
      outputFileSync(join(testDir, 'array.js'), 'module.exports = [() => {}]', 'utf-8')
      outputFileSync(join(testDir, 'not-valid.js'), 'sdfg', 'utf-8')
      const routes = [
         { method: 'get', url: 'valid', pathes: [join(testDir, 'valid.js')] },
         { method: 'get', url: 'array', pathes: [join(testDir, 'array.js')] },
         { method: 'get', url: 'not-valid', pathes: [join(testDir, 'not-valid.js')] },
         { method: 'get', url: ':not-valid-url', pathes: [] },
      ]
      const errors = []
      const result = prepareRoutes(routes, '/', errors)
      assert(errors.length === 2)
      assert(result.length === 2)
      assert(result.every(({ fns }) => Array.isArray(fns)));
      assert(errors.every((err) => err instanceof Error));
      removeSync(testDir)
   })

   it('should handle routes with empty pathes correctly', () => {
      const routes = [{ method: 'get', url: 'empty', pathes: [] }];
      const errors = [];
      const result = prepareRoutes(routes, '/', errors);

      assert.strictEqual(errors.length, 0);
      assert.strictEqual(result.length, 1);
      assert.strictEqual(result[0].fns.length, 0);
   });


   it('should correctly prepend urlPath to each route', () => {
      outputFileSync(join(testDir, 'valid.js'), 'module.exports = () => {}', 'utf-8')
      
      const routes = [{ method: 'get', url: 'test', pathes: [join(testDir, 'valid.js')] }];
      const errors = [];
      const result = prepareRoutes(routes, '/api', errors);
      
      assert.strictEqual(errors.length, 0);
      assert.strictEqual(result.length, 1);
      assert.strictEqual(result[0].path, '/api/test');
      removeSync(testDir)
  });
  
});
